<?php

class AAGK_Media_Download
{
    // Early plugin initialization.
    public static function bootstrap()
    {
        add_action( 'admin_init', array( __CLASS__, 'admin_init' ) );
        add_action( 'admin_bar_menu', function ( &$wp_admin_bar ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'appearance',
                'id'     => 'media-library',
                'title'  => __( 'Media' ),
                'href'   => admin_url( '/upload.php' ),
            ) );
        } );
    }
    
    public static function is_upload_screen_admin()
    {
        if ( !function_exists( 'get_current_screen' ) || !($screen = get_current_screen()) || $screen->base != 'upload' ) {
            return false;
        }
        return true;
    }
    
    public static function admin_init()
    {
        add_action( 'print_media_templates', array( __CLASS__, 'inject_js' ) );
        add_action( 'admin_notices', function () {
            if ( !self::is_upload_screen_admin() ) {
                return;
            }
            if ( defined( 'DISABLE_NAG_NOTICES' ) && DISABLE_NAG_NOTICES ) {
                return;
            }
            if ( self::f()->can_use_premium_code() ) {
                return;
            }
            if ( get_transient( 'mldf-upgrade-nag' ) ) {
                return;
            }
            ?>
				<div class="notice notice-info is-dismissible notice-mldf">
					<h3><?php 
            _e( 'Thanks for using Media Library File Download!', 'media-download' );
            ?></h3>
					<p><?php 
            printf( __( 'Want more features? Upgrade to <a href="%s">Pro</a>', 'media-download' ), esc_attr( self::f()->get_upgrade_url() ) );
            ?></p>
					<ul>
						<!-- <li><//?php _e( 'Image conversion', 'media-download' ); ?></li> -->
						<li><?php 
            _e( 'Bulk downloads', 'media-download' );
            ?></li>
						<li><?php 
            _e( 'Unlimited downloads', 'media-download' );
            ?></li>
						<li><?php 
            _e( 'Download from thumbnail', 'media-download' );
            ?></li>
					</ul>
				</div>
			<?php 
        } );
        add_action( 'admin_enqueue_scripts', function () {
            if ( !self::is_upload_screen_admin() ) {
                return;
            }
            if ( !current_user_can( 'upload_files' ) ) {
                return;
            }
            add_thickbox();
            wp_enqueue_script( 'aagk-media-download', plugins_url( 'media-download.js', __FILE__ ), array( 'jquery', 'thickbox' ) );
            wp_enqueue_style( 'aagk-media-download', plugins_url( 'media-download.css', __FILE__ ) );
            wp_localize_script( 'aagk-media-download', 'AAGK', array(
                'MLFD' => array(
                'endpoint'  => add_query_arg( 'mlfd-action', 'download', admin_url( 'upload.php' ) ),
                'downloads' => get_option( 'aagk-mfld-downloads', 0 ),
                'softlimit' => 10,
                'ispro'     => self::f()->can_use_premium_code(),
                'nag'       => array(
                'title'    => __( 'Nice!', 'media-download' ),
                'subtitle' => __( 'You have downloaded %d files!', 'media-download' ),
                'message'  => __( "If you're enjoying this plugin consider upgrading to PRO:", 'media-download' ),
                'features' => array( __( 'Unlimited downloads', 'media-download' ), __( 'Bulk downloads', 'media-download' ), __( 'Download from thumbnail', 'media-download' ) ),
                'upgrade'  => __( 'Upgrade now', 'media-download' ),
                'url'      => self::f()->get_upgrade_url(),
            ),
                'ui'        => array(
                'download'    => __( 'Download Archive', 'media-download' ),
                'replace'     => __( 'Replace', 'media-download' ),
                'replacefile' => __( 'Replace the current file with one of the same type', 'media-download' ),
            ),
            ),
            ) );
        } );
        self::handle_download_request();
        self::handle_replace_request();
        self::handle_notice_dimiss();
        add_filter(
            'plugin_action_links_' . self::f()->get_plugin_basename(),
            function (
            $actions,
            $plugin_file,
            $plugin_data,
            $context
        ) {
            
            if ( !self::f()->can_use_premium_code() ) {
                $style = 'font-weight: bold; color: #a00;';
                $actions['go-pro media-download-free'] = sprintf( '<a style="%s" href="%s">Go Pro</a>', $style, esc_url( self::f()->get_upgrade_url() ) );
            }
            
            return $actions;
        },
            10,
            4
        );
    }
    
    public static function inject_js()
    {
        // Maybe one day we can use proper injection
        // https://core.trac.wordpress.org/ticket/50185
        if ( !current_user_can( 'upload_files' ) ) {
            return;
        }
        ?>
			<script>
				(function() {
					// Alter the attachment view template files.
					var original   = jQuery( '#tmpl-attachment-details' ).html();
					var original_2 = jQuery( '#tmpl-attachment-details-two-column' ).html();

					var prefix_download = '<button type="button" class="mlfd-download button media-button button-primary button-large"><?php 
        esc_html_e( 'Download', 'media-download' );
        ?></button>';

					var prefix_replace = '<button type="button" class="mlfd-replace button media-button button-primary button-large"><?php 
        esc_html_e( 'Replace', 'media-download' );
        ?></button>';

					// TODO: What about 1-column?

					jQuery( '#tmpl-attachment-details-two-column' ).html(
						original_2.replace( '<div class="compat-meta">',

							'<div class="media-download">' + prefix_download + prefix_replace + '</div'

							+ '<div class="compat-meta">' )
					);
				}());
			</script>
		<?php 
    }
    
    // The download handler.
    private static function handle_download_request()
    {
        if ( empty($_GET['mlfd-action']) ) {
            return;
        }
        if ( empty($_GET['id']) ) {
            return;
        }
        if ( 'download' !== $_GET['mlfd-action'] ) {
            return;
        }
        if ( !current_user_can( 'upload_files' ) ) {
            return;
        }
        if ( !($attachment = get_post( $_GET['id'] )) ) {
            return;
        }
        if ( 'attachment' !== $attachment->post_type ) {
            return;
        }
        if ( !($path = get_attached_file( $attachment->ID )) ) {
            return;
        }
        update_option( 'aagk-mfld-downloads', get_option( 'aagk-mfld-downloads', 0 ) + 1 );
        header( 'Content-Type: ' . $attachment->post_mime_type );
        header( 'Content-Length: ' . filesize( $path ) );
        header( 'Content-Disposition: attachment; filename=' . basename( $path ) );
        header( 'Content-Transfer-Encoding: binary' );
        header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
        readfile( $path );
        exit;
    }
    
    // Simple nag disable.
    private static function handle_notice_dimiss()
    {
        if ( isset( $_POST['notice-mldf-dismiss'] ) && current_user_can( 'upload_files' ) ) {
            set_transient( 'mldf-upgrade-nag', true, MONTH_IN_SECONDS );
        }
    }
    
    // The simple replace handler.
    private static function handle_replace_request()
    {
        if ( empty($_POST['mlfd-replace-id']) ) {
            return;
        }
        if ( empty($_FILES['mlfd-replace-file']) ) {
            return;
        }
        if ( !current_user_can( 'upload_files' ) ) {
            return;
        }
        if ( !($attachment = get_post( $_POST['mlfd-replace-id'] )) ) {
            return;
        }
        if ( 'attachment' !== $attachment->post_type ) {
            return;
        }
        if ( !($path = get_attached_file( $attachment->ID )) ) {
            return;
        }
        if ( $_FILES['mlfd-replace-file']['type'] != $attachment->post_mime_type ) {
            wp_die( __( 'Mime type mismatch. Please upload a file of the same type.', 'media-download' ) );
        }
        if ( !wp_check_filetype( $_FILES['mlfd-replace-file']['name'] ) ) {
            wp_die( __( 'Mime type mismatch. Please upload a file of the same type.', 'media-download' ) );
        }
        if ( !move_uploaded_file( $_FILES['mlfd-replace-file']['tmp_name'], $path ) ) {
            wp_die( __( 'Could not replace file.', 'media-download' ) );
        }
        
        if ( !class_exists( 'RegenerateThumbnails_Regenerator' ) ) {
            require_once __DIR__ . '/lib/regenerate-thumbnails/class-regeneratethumbnails-regenerator.php';
            $regenerator = RegenerateThumbnails_Regenerator::get_instance( $attachment->ID );
            $regenerator->regenerate( array(
                'only_regenerate_missing_thumbnails' => false,
            ) );
            $regenerator->update_usages_in_posts();
        }
    
    }
    
    // Return Freemius object.
    private static function f()
    {
        return aagk_mlfd_fs();
    }

}